"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Agent = void 0;
const rxjs_1 = require("rxjs");
const constants_1 = require("../constants");
const crypto_1 = require("../crypto");
const JwsService_1 = require("../crypto/JwsService");
const error_1 = require("../error");
const plugins_1 = require("../plugins");
const storage_1 = require("../storage");
const AgentConfig_1 = require("./AgentConfig");
const AgentModules_1 = require("./AgentModules");
const BaseAgent_1 = require("./BaseAgent");
const EventEmitter_1 = require("./EventEmitter");
const context_1 = require("./context");
// Any makes sure you can use Agent as a type without always needing to specify the exact generics for the agent
// biome-ignore lint/suspicious/noExplicitAny: <explanation>
class Agent extends BaseAgent_1.BaseAgent {
    constructor(options, dependencyManager = new plugins_1.DependencyManager()) {
        const agentConfig = new AgentConfig_1.AgentConfig(options.config, options.dependencies);
        const modulesWithDefaultModules = (0, AgentModules_1.extendModulesWithDefaultModules)(options.modules);
        // Register internal dependencies
        dependencyManager.registerSingleton(EventEmitter_1.EventEmitter);
        dependencyManager.registerSingleton(JwsService_1.JwsService);
        dependencyManager.registerSingleton(storage_1.StorageVersionRepository);
        dependencyManager.registerSingleton(storage_1.StorageUpdateService);
        // This is a really ugly hack to make tsyringe work without any SigningProviders registered
        // It is currently impossible to use @injectAll if there are no instances registered for the
        // token. We register a value of `default` by default and will filter that out in the registry.
        // Once we have a signing provider that should always be registered we can remove this. We can make an ed25519
        // signer using the @stablelib/ed25519 library.
        dependencyManager.registerInstance(crypto_1.SigningProviderToken, 'default');
        dependencyManager.registerInstance(AgentConfig_1.AgentConfig, agentConfig);
        dependencyManager.registerInstance(constants_1.InjectionSymbols.AgentDependencies, agentConfig.agentDependencies);
        dependencyManager.registerInstance(constants_1.InjectionSymbols.Stop$, new rxjs_1.Subject());
        dependencyManager.registerInstance(constants_1.InjectionSymbols.FileSystem, new agentConfig.agentDependencies.FileSystem());
        // Register all modules. This will also include the default modules
        dependencyManager.registerModules(modulesWithDefaultModules);
        // Register possibly already defined services
        if (!dependencyManager.isRegistered(constants_1.InjectionSymbols.Wallet)) {
            throw new error_1.CredoError("Missing required dependency: 'Wallet'. You can register it using the AskarModule, or implement your own.");
        }
        if (!dependencyManager.isRegistered(constants_1.InjectionSymbols.Logger)) {
            dependencyManager.registerInstance(constants_1.InjectionSymbols.Logger, agentConfig.logger);
        }
        if (!dependencyManager.isRegistered(constants_1.InjectionSymbols.StorageService)) {
            throw new error_1.CredoError("Missing required dependency: 'StorageService'. You can register it using the AskarModule, or implement your own.");
        }
        // TODO: contextCorrelationId for base wallet
        // Bind the default agent context to the container for use in modules etc.
        dependencyManager.registerInstance(context_1.AgentContext, new context_1.AgentContext({
            dependencyManager,
            contextCorrelationId: 'default',
        }));
        // If no agent context provider has been registered we use the default agent context provider.
        if (!dependencyManager.isRegistered(constants_1.InjectionSymbols.AgentContextProvider)) {
            dependencyManager.registerSingleton(constants_1.InjectionSymbols.AgentContextProvider, context_1.DefaultAgentContextProvider);
        }
        super(agentConfig, dependencyManager);
    }
    get events() {
        return this.eventEmitter;
    }
    async initialize() {
        await super.initialize();
        for (const [, module] of Object.entries(this.dependencyManager.registeredModules)) {
            if (module.initialize) {
                await module.initialize(this.agentContext);
            }
        }
        this._isInitialized = true;
    }
    async shutdown() {
        const stop$ = this.dependencyManager.resolve(constants_1.InjectionSymbols.Stop$);
        // All observables use takeUntil with the stop$ observable
        // this means all observables will stop running if a value is emitted on this observable
        stop$.next(true);
        for (const [, module] of Object.entries(this.dependencyManager.registeredModules)) {
            if (module.shutdown) {
                await module.shutdown(this.agentContext);
            }
        }
        if (this.wallet.isInitialized) {
            await this.wallet.close();
        }
        this._isInitialized = false;
    }
}
exports.Agent = Agent;
//# sourceMappingURL=Agent.js.map