"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Key = void 0;
const ec_compression_1 = require("ec-compression");
const utils_1 = require("../utils");
const keyUtils_1 = require("./keyUtils");
const multiCodecKey_1 = require("./multiCodecKey");
class Key {
    constructor(publicKey, keyType) {
        this.publicKey = (0, ec_compression_1.decompressPublicKeyIfPossible)(publicKey, keyType);
        this.keyType = keyType;
    }
    get compressedPublicKey() {
        return (0, ec_compression_1.compressPublicKeyIfPossible)(this.publicKey, this.keyType);
    }
    static fromPublicKey(publicKey, keyType) {
        return new Key(publicKey, keyType);
    }
    static fromPublicKeyBase58(publicKey, keyType) {
        const publicKeyBytes = Uint8Array.from(utils_1.TypedArrayEncoder.fromBase58(publicKey));
        return Key.fromPublicKey(publicKeyBytes, keyType);
    }
    static fromFingerprint(fingerprint) {
        const { data } = utils_1.MultiBaseEncoder.decode(fingerprint);
        const [code, byteLength] = utils_1.VarintEncoder.decode(data);
        const publicKey = data.slice(byteLength);
        const keyType = (0, multiCodecKey_1.getKeyTypeByMultiCodecPrefix)(code);
        return new Key(publicKey, keyType);
    }
    get prefixedPublicKey() {
        const multiCodecPrefix = (0, multiCodecKey_1.getMultiCodecPrefixByKeyType)(this.keyType);
        // Create Buffer with length of the prefix bytes, then use varint to fill the prefix bytes
        const prefixBytes = utils_1.VarintEncoder.encode(multiCodecPrefix);
        // Combine prefix with public key
        // Multicodec requires compressable keys to be compressed
        return new Uint8Array([...prefixBytes, ...this.compressedPublicKey]);
    }
    get fingerprint() {
        return `z${utils_1.TypedArrayEncoder.toBase58(this.prefixedPublicKey)}`;
    }
    get publicKeyBase58() {
        return utils_1.TypedArrayEncoder.toBase58(this.publicKey);
    }
    get supportsEncrypting() {
        return (0, keyUtils_1.isEncryptionSupportedForKeyType)(this.keyType);
    }
    get supportsSigning() {
        return (0, keyUtils_1.isSigningSupportedForKeyType)(this.keyType);
    }
    // We return an object structure based on the key, so that when this object is
    // serialized to JSON it will be nicely formatted instead of the bytes printed
    toJSON() {
        return {
            keyType: this.keyType,
            publicKeyBase58: this.publicKeyBase58,
        };
    }
}
exports.Key = Key;
//# sourceMappingURL=Key.js.map